# -*- coding: utf-8 -*-
from iosanita.contenttypes.interfaces import IoSanitaViewExtraData
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.services import Service
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class ViewExtraData(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        result = {
            "view-extra-data": {
                "@id": f"{self.context.absolute_url()}/@view-extra-data"
            }
        }
        data = queryMultiAdapter((self.context, self.request), IoSanitaViewExtraData)()
        result["view-extra-data"].update(data)

        return result


class ViewExtraDataGet(Service):
    def reply(self):
        data = ViewExtraData(self.context, self.request)
        return data(expand=True)["view-extra-data"]



# -*- coding: utf-8 -*-
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer


@indexer(IDexterityContent)
def uo_correlata_uid(context, **kw):
    """ """
    return [
        x.to_object.UID()
        for x in getattr(context.aq_base, "uo_correlata", [])
        if x.to_object
    ]


@indexer(IDexterityContent)
def struttura_correlata_uid(context, **kw):
    """ """
    return [
        x.to_object.UID()
        for x in getattr(context.aq_base, "struttura_correlata", [])
        if x.to_object
    ]


@indexer(IDexterityContent)
def servizio_correlato_uid(context, **kw):
    """ """
    return [
        x.to_object.UID()
        for x in getattr(context.aq_base, "servizio_correlato", [])
        if x.to_object
    ]



from collective.taxonomy import PATH_SEPARATOR
from iosanita.contenttypes.indexers.taxonomies import get_taxonomy_vocab
from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from iosanita.contenttypes.interfaces.punto_di_contatto import IPuntoDiContatto
from plone import api
from plone.restapi.interfaces import IJSONSummarySerializerMetadata
from plone.restapi.interfaces import ISerializeToJsonSummary
from redturtle.volto.restapi.serializer.summary import DefaultJSONSummarySerializer
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface

import logging


logger = logging.getLogger(__name__)


@implementer(IJSONSummarySerializerMetadata)
class JSONSummarySerializerMetadata:
    def default_metadata_fields(self):
        """
        Force always return some metadata
        """
        return {
            "parliamo_di_metadata",
            "a_chi_si_rivolge_tassonomia_metadata",
            "id",
            "tipologia_notizia",
            "start",
            "end",
            "recurrence",
            "whole_day",
            "open_end",
            "street",
            "zip_code",
            "city",
            "provincia",
            "geolocation",
            "servizio_attivo",
            "parent",
            "Date",
            "incarico_metadata",
            "modified",
        }


@implementer(ISerializeToJsonSummary)
@adapter(Interface, IIosanitaContenttypesLayer)
class IOSanitaJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        """
        Customize type_title for News Items
        """
        data = super().__call__(force_all_metadata=force_all_metadata)
        obj = None
        if self.is_get_call() or data["@type"] == "Bando":
            obj = self.get_real_object()
        if self.is_get_call():
            data["has_children"] = self.has_children(obj=obj)
        if data["@type"] == "News Item":
            data["type_title"] = self.get_news_type_title()
        if data["@type"] == "Bando":
            metadata_fields = self.metadata_fields()
            if "bando_state" in metadata_fields or self.show_all_metadata_fields:
                data["bando_state"] = self.get_bando_state(obj=obj)
        return data

    def get_real_object(self):
        try:
            return self.context.getObject()
        except AttributeError:
            return self.context

    def get_news_type_title(self):
        tipologia_notizia = getattr(self.context, "tipologia_notizia")
        if tipologia_notizia:
            tipologia_notizia = tipologia_notizia[0]
            taxonomy_vocab = get_taxonomy_vocab("tipologia_notizia")
            taxonomy_value = taxonomy_vocab.inv_data.get(tipologia_notizia, None)
            if taxonomy_value:
                return taxonomy_value.replace(PATH_SEPARATOR, "", 1)
        return "Notizia"

    def is_get_call(self):
        if self.request.get("other", {}).get("method", "") == "GET":
            return True
        if getattr(self.request, "environ", {}).get("REQUEST_METHOD", "") == "GET":
            return True
        return False

    def has_children(self, obj):
        """
        Return info if the item has at least one child
        """
        try:
            if obj.aq_base.keys():
                return True
        except AttributeError:
            return False
        return False

    def get_bando_state(self, obj):
        bando_view = api.content.get_view(
            "bando_view", context=obj, request=self.request
        )
        return bando_view.getBandoState()


@implementer(ISerializeToJsonSummary)
@adapter(IPuntoDiContatto, IIosanitaContenttypesLayer)
class PuntoDiContattoJSONSummarySerializer(IOSanitaJSONSummarySerializer):
    def __call__(self):
        data = super().__call__()

        data["contatti"] = getattr(self.context, "contatti", [])

        return data



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import unittest


class TestDocumentoSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_documento(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Documento"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.categorization",
                "plone.basic",
                "plone.locking",
                "plone.constraintypes",
                "plone.leadimage",
                "volto.preview_image",
                "plone.relateditems",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "iosanita.contenttypes.behavior.a_chi_si_rivolge",
                "collective.taxonomy.generated.a_chi_si_rivolge_tassonomia",
                "collective.taxonomy.generated.parliamo_di",
            ),
        )

    def test_documento_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Documento").json()
        self.assertEqual(len(resp["fieldsets"]), 9)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cosa_e",
                "riferimenti",
                "a_chi_si_rivolge",
                "settings",
                "ownership",
                "dates",
                "categorization",
                "seo",
            ],
        )

    def test_documento_required_fields(self):
        resp = self.api_session.get("@types/Documento").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                    "descrizione_estesa",
                    "uo_correlata",
                ]
            ),
        )

    def test_documento_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Documento").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "protocollo",
                "data_protocollo",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
                "parliamo_di",
            ],
        )

    def test_documento_cosa_e_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Documento").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            ["descrizione_estesa"],
        )

    def test_documento_riferimenti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Documento").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            [
                "servizio_procedura_riferimento",
                "uo_correlata",
                "autori",
            ],
        )

    def test_documento_fields_a_chi_si_rivolge_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Documento").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            ["a_chi_si_rivolge", "a_chi_si_rivolge_tassonomia"],
        )


class TestDocumento(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_documento_default_children(self):
        documento = api.content.create(
            container=self.portal, type="Documento", title="xxx"
        )

        self.assertEqual(documento.keys(), ["immagini"])

    def test_documento_immagini_has_filtered_addable_types(self):
        documento = api.content.create(
            container=self.portal, type="Documento", title="xxx"
        )
        immagini = ISelectableConstrainTypes(documento["immagini"])
        self.assertEqual(immagini.getConstrainTypesMode(), 1)
        self.assertEqual(immagini.getLocallyAllowedTypes(), ["Link", "Image"])



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.event import notify
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import ObjectModifiedEvent

import unittest


class TestUOCorrelata(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="uo"
        )

    def test_uo_correlata_reference_is_in_catalog(self):
        """ """
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test servizio",
        )
        intids = getUtility(IIntIds)
        struttura.uo_correlata = [RelationValue(intids.getId(self.uo))]
        notify(ObjectModifiedEvent(struttura))

        res = api.content.find(uo_correlata_uid=self.uo.UID())

        self.assertEqual(len(res), 1)
        self.assertEqual(res[0].UID, struttura.UID())



from plone.app.dexterity import _
from plone.app.dexterity import textindexer
from plone.app.dexterity.behaviors.metadata import Basic
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from z3c.form.interfaces import IAddForm
from z3c.form.interfaces import IEditForm
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class IIoSanitaBasic(model.Schema):
    """
    Make description required
    """

    # default fieldset
    title = schema.TextLine(title=_("label_title", default="Title"), required=True)

    description = schema.Text(
        title=_("label_description", default="Summary"),
        description=_(
            "help_description", default="Used in item listings and search results."
        ),
        required=True,
        missing_value="",
    )

    directives.order_before(description="*")
    directives.order_before(title="*")

    directives.omitted("title", "description")
    directives.no_omit(IEditForm, "title", "description")
    directives.no_omit(IAddForm, "title", "description")

    textindexer.searchable("title")
    textindexer.searchable("description")


class IoSanitaBasic(Basic):
    """ """



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.namedfile import field
from plone.supermodel import model
from z3c.form.interfaces import IAddForm
from z3c.form.interfaces import IEditForm
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IPersona(model.Schema, IIosanitaContenttypes):
    """Marker interface for contenttype Persona"""

    cognome = schema.TextLine(
        title=_("cognome_label", default="Cognome"),
        description="",
        required=True,
        missing_value="",
        default="",
    )
    nome = schema.TextLine(
        title=_("nome_label", default="Nome"),
        description="",
        required=True,
        missing_value="",
        default="",
    )
    titolo_persona = schema.TextLine(
        title=_("titolo_persona_label", default="Titolo persona"),
        description=_(
            "titolo_persona_help", default="Un titolo personale come ad esempio Dr. "
        ),
        required=False,
        missing_value="",
        default="",
    )
    title = schema.TextLine(
        title=_("label_title", default="Titolo"),
        description="",
        required=False,
        missing_value="",
        readonly=True,
        default="",
    )
    description = schema.Text(
        title=_("label_description", default="Descrizione"),
        description=_(
            "help_description",
            default="Usato nell'elenco degli elementi e nei risultati delle ricerche.",
        ),
        required=False,
        missing_value="",
    )

    image = field.NamedBlobImage(
        title=_("foto_persona_label", default="Foto"),
        required=False,
        description=_(
            "foto_persona_help",
            default="Foto da mostrare dentro al sito. "
            "La dimensione suggerita è 100x180px.",
        ),
    )

    altri_incarichi = BlocksField(
        title=_("altri_incarichi_label", default="Altri incarichi"),
        description=_(
            "altri incarichi_help",
            default="Indicazione degli altri incarichi della persona all'interno dell'ASL.",
        ),
        required=False,
    )

    competenze = BlocksField(
        title=_("competenze_label", default="Competenze"),
        description=_(
            "competenze_help",
            default="Descrizione del ruolo e dei compiti della persona.",
        ),
        required=True,
    )

    orari_ricevimento = BlocksField(
        title=_("orari_ricevimento_label", default="Orari di ricevimento"),
        description=_(
            "orari_ricevimento_help",
            default="Orari durante i quali è possibile incontrare la persona descritta, se effettua ricevimento.",
        ),
        required=False,
    )
    struttura_in_cui_opera = RelationList(
        title=_(
            "struttura_in_cui_opera_label",
            default="Strutture in cui opera",
        ),
        description=_(
            "struttura_in_cui_opera_help",
            default="Seleziona una lista di strutture in cui opera la persona. Necessario per le persone interne all'ASL, mentre è opzionale per i medici di base e pediatri.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        missing_value=(),
    )
    struttura_ricevimento = RelationList(
        title=_(
            "struttura_ricevimento_label",
            default="Struttura ricevimento",
        ),
        description=_(
            "struttura_ricevimento_help",
            default="Seleziona una struttura presente sul portale dove la persona riceve i pazienti. Se non è disponibile, compila i campi successivi.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        missing_value=(),
    )

    biografia = BlocksField(
        title=_("biografia_label", default="Biografia"),
        description=_("biografia_help", default=""),
        required=False,
    )

    # custom widgets
    form.widget(
        "struttura_ricevimento",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            # "maximumSelectionSize": 1,
            "selectableTypes": ["Struttura"],
        },
    )
    form.widget(
        "struttura_in_cui_opera",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Struttura"],
        },
    )

    # custom fieldsets
    model.fieldset(
        "incarichi",
        label=_("incarichi_label", default="Incarichi"),
        fields=[
            "altri_incarichi",
        ],
    )
    model.fieldset(
        "competenze",
        label=_("competenze_label", default="Competenze"),
        fields=[
            "competenze",
        ],
    )

    model.fieldset(
        "dove",
        label=_("dove_label", default="Dove"),
        description=_(
            "dove_persona_help",
            default="Se la persona fa ricevimento, indicare dove. "
            "Questo elemento è necessario per i medici di base e pediatri.",
        ),
        fields=["struttura_ricevimento", "struttura_in_cui_opera"],
    )

    model.fieldset(
        "orari_ricevimento",
        label=_("orari_ricevimento_label", default="Orari di ricevimento"),
        fields=[
            "orari_ricevimento",
        ],
    )

    model.fieldset(
        "biografia",
        label=_("biografia_label", default="Biografia"),
        fields=[
            "biografia",
        ],
    )

    form.order_before(description="*")
    form.order_before(title="*")
    form.order_before(titolo_persona="*")
    form.order_before(nome="*")
    form.order_before(cognome="*")
    form.order_after(altri_incarichi="incarico")

    form.omitted("description")
    form.omitted("title")
    form.no_omit(IEditForm, "description")
    form.no_omit(IAddForm, "description")

    # SearchableText fields
    textindexer.searchable("nome")
    textindexer.searchable("cognome")
    textindexer.searchable("competenze")



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from transaction import commit

import json
import unittest


class TestCustomValidation(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.request["LANGUAGE"] = "it"

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_event_raise_badrequest_if_missing_organizzato_da(self):
        api.content.create(type="PuntoDiContatto", title="pdc", container=self.portal)
        commit()
        data = {
            "costo": {"blocks": {}},
            "description": "asdasd",
            "descrizione_estesa": {"blocks": {}},
            "end": "2024-07-26T10:00:00+00:00",
            "geolocation": {"latitude": 44.821502405601, "longitude": 11.632933286779},
            "pdc_correlato": [{"@id": "/pdc"}],
            "start": "2024-07-26T09:00:00+00:00",
            "tipologia_evento": "giornata-informativa",
            "title": "titolo",
            "provincia": "xxx",
            "@type": "Event",
            "a_chi_si_rivolge": {
                "blocks": {
                    "b8850749-4d5f-4b5f-b842-c050bcf7fd6f": {
                        "@type": "slate",
                        "plaintext": "xxx",
                    }
                },
            },
        }
        resp = self.api_session.post(self.portal_url, json=data)
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(
            json.loads(resp.json()["message"]),
            [
                {
                    "field": "organizzato_da_esterno",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "field": "organizzato_da_interno",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "message": 'Devi compilare almeno uno dei due campi per "Organizzato da" nel tab "Contatti".'
                },
            ],
        )

        # organizzato_da_esterno is considered empty block
        data.update(
            {
                "organizzato_da_esterno": {
                    "blocks": {
                        "b8850749-4d5f-4b5f-b842-c050bcf7fd6f": {"@type": "slate"}
                    },
                    "blocks_layout": {
                        "items": ["b8850749-4d5f-4b5f-b842-c050bcf7fd6f"]
                    },
                },
                "organizzato_da_interno": [],
            }
        )

        resp = self.api_session.post(self.portal_url, json=data)
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(
            json.loads(resp.json()["message"]),
            [
                {
                    "field": "organizzato_da_esterno",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "field": "organizzato_da_interno",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "message": 'Devi compilare almeno uno dei due campi per "Organizzato da" nel tab "Contatti".'
                },
            ],
        )

        # now compile only organizzato_da_interno
        data["organizzato_da_interno"] = [{"@id": "/pdc"}]
        resp = self.api_session.post(self.portal_url, json=data)
        self.assertEqual(resp.status_code, 201)

        # and compile only organizzato_da_esterno
        data["organizzato_da_interno"] = []
        data["organizzato_da_esterno"] = {
            "blocks": {
                "b8850749-4d5f-4b5f-b842-c050bcf7fd6f": {
                    "@type": "slate",
                    "plaintext": "xxx",
                }
            },
        }
        resp = self.api_session.post(self.portal_url, json=data)
        self.assertEqual(resp.status_code, 201)

    def test_raise_bad_request_if_missing_a_chi_si_rivolge_fields(self):
        api.content.create(type="PuntoDiContatto", title="pdc", container=self.portal)
        commit()
        data = {
            "come_accedere": {"blocks": {}},
            "orari_apertura": {"blocks": {}},
            "description": "asdasd",
            "pdc_correlato": [{"@id": "/pdc"}],
            "provincia": "xxx",
            "city": "xxx",
            "country": "380",
            "street": "xxx",
            "zip_code": "xxx",
            "title": "titolo",
            "@type": "Struttura",
            "geolocation": {"latitude": 1.0, "longitude": 1.0},
        }
        resp = self.api_session.post(self.portal_url, json=data)

        self.assertEqual(resp.status_code, 400)
        self.assertEqual(
            json.loads(resp.json()["message"]),
            [
                {
                    "field": "a_chi_si_rivolge",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "field": "a_chi_si_rivolge_tassonomia",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "message": 'Devi compilare almeno uno dei due campi del tab "A chi si rivolge".'
                },
            ],
        )

        # a_chi_si_rivolge is considered empty block
        data.update(
            {
                "a_chi_si_rivolge": {
                    "blocks": {
                        "b8850749-4d5f-4b5f-b842-c050bcf7fd6f": {"@type": "slate"}
                    },
                    "blocks_layout": {
                        "items": ["b8850749-4d5f-4b5f-b842-c050bcf7fd6f"]
                    },
                },
                "a_chi_si_rivolge_tassonomia": [],
            }
        )

        resp = self.api_session.post(self.portal_url, json=data)
        self.assertEqual(resp.status_code, 400)

        self.assertEqual(
            json.loads(resp.json()["message"]),
            [
                {
                    "field": "a_chi_si_rivolge",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "field": "a_chi_si_rivolge_tassonomia",
                    "message": "Compila almeno uno di questi campi.",
                },
                {
                    "message": 'Devi compilare almeno uno dei due campi del tab "A chi si rivolge".'
                },
            ],
        )

        # now compile only a_chi_si_rivolge_tassonomia
        data["a_chi_si_rivolge_tassonomia"] = ["anziani"]
        resp = self.api_session.post(self.portal_url, json=data)
        self.assertEqual(resp.status_code, 201)

        # and compile only a_chi_si_rivolge
        data["a_chi_si_rivolge_tassonomia"] = []
        data["a_chi_si_rivolge"] = {
            "blocks": {
                "b8850749-4d5f-4b5f-b842-c050bcf7fd6f": {
                    "@type": "slate",
                    "plaintext": "xxx",
                }
            },
        }
        resp = self.api_session.post(self.portal_url, json=data)
        self.assertEqual(resp.status_code, 201)

    def test_raise_bad_request_on_struttura_if_missing_dove_fields(self):
        api.content.create(type="PuntoDiContatto", title="pdc", container=self.portal)
        commit()
        data = {
            "come_accedere": {"blocks": {}},
            "orari_apertura": {"blocks": {}},
            "description": "asdasd",
            "pdc_correlato": [{"@id": "/pdc"}],
            "title": "titolo",
            "@type": "Struttura",
            "a_chi_si_rivolge_tassonomia": ["anziani"],
        }
        resp = self.api_session.post(self.portal_url, json=data)

        self.assertEqual(resp.status_code, 400)
        self.assertEqual(
            json.loads(resp.json()["message"]),
            [
                {
                    "field": "street",
                    "message": "Devi compilare questi campi per poter cercare la posizione in mappa.",
                },
                {
                    "field": "city",
                    "message": "Devi compilare questi campi per poter cercare la posizione in mappa.",
                },
                {
                    "message": 'Devi compilare almeno uno dei due campi del tab "Dove" e impostare la Geolocation.'
                },
            ],
        )



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.autoform import directives as form
from plone.supermodel import model


class IComeFarePer(model.Schema, IIosanitaContenttypes):
    """Marker interface for content type"""

    panoramica = BlocksField(
        title=_("panoramica_label", default="Panoramica"),
        description=_(
            "panoramica_help",
            default="Descrizione più estesa della procedura di cui viene descritto il processo di usufruizione.",
        ),
        required=True,
    )
    come_fare = BlocksField(
        title=_("come_fare_label", default="Come fare"),
        description=_(
            "come_fare_help",
            default="Descrizione generale del processo da seguire per effettuare la procedura."
            ' Se il processo è composto da più step, allora vanno creati dei contenuti di tipo "Step" con informazioni dettagliate.',
        ),
        required=True,
    )
    form.order_after(panoramica="sottotitolo")

    # custom fieldsets
    model.fieldset(
        "come_fare",
        label=_("come_fare_label", default="Come fare"),
        fields=[
            "come_fare",
        ],
    )



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IAChiSiRivolge(model.Schema):
    """Marker interface"""

    a_chi_si_rivolge = BlocksField(
        title=_("a_chi_si_rivolge_label", default="A chi si rivolge"),
        required=False,
        description=_(
            "a_chi_e_rivolta_help",
            default="Descrizione testuale degli utenti dell'ASL a cui è rivolto questo contenuto.",
        ),
    )
    model.fieldset(
        "a_chi_si_rivolge",
        label=_("a_chi_si_rivolge_label", default="A chi si rivolge"),
        description=_(
            "a_chi_si_rivolge_fieldset_help",
            default="Compila almeno uno dei due campi.",
        ),
        fields=["a_chi_si_rivolge"],
    )


@implementer(IAChiSiRivolge)
@adapter(IDexterityContent)
class AChiSiRivolge(object):
    """"""

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-

from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList


class IStruttura(model.Schema, IIosanitaContenttypes):
    """Marker interface for content type"""

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa_struttura_label", default="Descrizione estesa"),
        required=False,
        description=_(
            "descrizione_estesa_struttura_help",
            default="Descrizione più estesa della struttura con riferimento alle principali attività sanitarie svolte.",
        ),
    )

    come_accedere = BlocksField(
        title=_("come_accedere_label", default="Come accedere"),
        required=True,
        description=_(
            "come_accedere_struttura_help",
            default="Modalità di accesso alla struttura con particolare attenzione agli accessi per disabili ed eventuale descrizione di come arrivare, costi e regole di accesso.",
        ),
    )

    orari = BlocksField(
        title=_("orari_struttura_label", default="Orari di apertura"),
        required=True,
        description=_(
            "orari_struttura_help",
            default="Orari di apertura della struttura al pubblico.",
        ),
    )

    responsabile_correlato = RelationList(
        title=_(
            "responsabile_correlato_struttura_label",
            default="Responsabile",
        ),
        description=_(
            "responsabile_correlato_struttura_help",
            default="La persona che dirige la struttura.",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
    )

    coordinatore_correlato = RelationList(
        title=_(
            "coordinatore_correlato_struttura_label",
            default="Coordinatore",
        ),
        description=_(
            "coordinatore_correlato_struttura_help",
            default="La persona che coordina la struttura.",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
    )
    uo_correlata = RelationList(
        title=_(
            "uo_correlata_struttura_label",
            default="Unità organizzativa di appartenenza",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
    )
    struttura_correlata = RelationList(
        title=_(
            "struttura_correlata_struttura_label",
            default="Struttura correlata",
        ),
        description=_(
            "struttura_correlata_struttura_help",
            default="Seleziona una struttura correlata.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        missing_value=(),
    )

    # widgets
    form.widget(
        "struttura_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Struttura"],
        },
    )
    form.widget(
        "responsabile_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"selectableTypes": ["Persona"]},
    )
    form.widget(
        "coordinatore_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"selectableTypes": ["Persona"]},
    )

    form.widget(
        "uo_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"selectableTypes": ["UnitaOrganizzativa"]},
    )

    # fieldsets
    model.fieldset(
        "cosa_e",
        label=_("cosa_e_fieldset", default="Cos'è"),
        fields=["descrizione_estesa"],
    )

    model.fieldset(
        "come_accedere",
        label=_("come_accedere_label", default="Come accedere"),
        fields=["come_accedere"],
    )
    model.fieldset(
        "orari",
        label=_("orari_label", default="Orari di apertura"),
        fields=["orari"],
    )

    model.fieldset(
        "persone_struttura",
        label=_("persone_struttura_label", default="Persone struttura"),
        fields=[
            "responsabile_correlato",
            "coordinatore_correlato",
        ],
    )
    model.fieldset(
        "contenuti_collegati",
        label=_("contenuti_collegati_label", default="Contenuti collegati"),
        fields=["uo_correlata", "struttura_correlata"],
    )



from copy import deepcopy
from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from iosanita.contenttypes.interfaces import IoSanitaMigrationMarker
from plone import api
from plone.dexterity.interfaces import IDexterityContent
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.restapi import _
from plone.restapi.deserializer import json_body
from plone.restapi.deserializer.dxcontent import DeserializeFromJson as BaseDeserializer
from plone.restapi.interfaces import IDeserializeFromJson
from zExceptions import BadRequest
from zope.component import adapter
from zope.interface import implementer

import json


@implementer(IDeserializeFromJson)
@adapter(IDexterityContent, IIosanitaContenttypesLayer)
class DeserializeFromJson(BaseDeserializer):
    def __call__(
        self, validate_all=False, data=None, create=False, mask_validation_errors=True
    ):
        if data is None:
            data = json_body(self.request)
        # iosanità validations
        self.validate_data_iosanita(data=data, create=create)

        return super().__call__(
            validate_all=validate_all,
            data=data,
            create=create,
            mask_validation_errors=mask_validation_errors,
        )

    @property
    def mutual_required_msg(self):
        return api.portal.translate(
            _(
                "mutual_required_field_msg",
                default="Compila almeno uno di questi campi.",
            )
        )

    def validate_data_iosanita(self, data, create):
        if IoSanitaMigrationMarker.providedBy(self.request):
            return
        if create:
            portal_type = data.get("@type", "")
        else:
            portal_type = self.context.portal_type
        self.validate_a_chi_si_rivolge(
            portal_type=portal_type, data=data, create=create
        )

        if portal_type == "Event":
            self.validate_event(data=data, create=create)
        if portal_type in ["Struttura", "UnitaOrganizzativa"]:
            self.validate_dove(data=data, create=create)

    def validate_a_chi_si_rivolge(self, portal_type, data, create):
        portal_types = api.portal.get_tool(name="portal_types")
        behaviors = portal_types[portal_type].behaviors

        has_behaviors = False
        for bhv in [
            "iosanita.contenttypes.behavior.a_chi_si_rivolge",
            "collective.taxonomy.generated.a_chi_si_rivolge_tassonomia",
        ]:
            if bhv in behaviors:
                has_behaviors = True
        if not has_behaviors:
            # skip check
            return

        has_a_chi_si_rivolge = self.has_field_value(
            data=data, create=create, field_id="a_chi_si_rivolge", is_block_field=True
        )
        has_a_chi_si_rivolge_tassonomia = self.has_field_value(
            data=data, create=create, field_id="a_chi_si_rivolge_tassonomia"
        )

        if not has_a_chi_si_rivolge and not has_a_chi_si_rivolge_tassonomia:
            raise BadRequest(
                json.dumps(
                    [
                        {
                            "field": "a_chi_si_rivolge",
                            "message": self.mutual_required_msg,
                        },
                        {
                            "field": "a_chi_si_rivolge_tassonomia",
                            "message": self.mutual_required_msg,
                        },
                        {
                            "message": api.portal.translate(
                                _(
                                    "a_chi_si_rivolge_validation_error",
                                    default='Devi compilare almeno uno dei due campi del tab "A chi si rivolge".',
                                )
                            )
                        },
                    ]
                )
            )

    def validate_event(self, data, create):
        # validate organizzato da

        has_organizzato_da_esterno = self.has_field_value(
            data=data,
            create=create,
            field_id="organizzato_da_esterno",
            is_block_field=True,
        )
        has_organizzato_da_interno = self.has_field_value(
            data=data, create=create, field_id="organizzato_da_interno"
        )

        if not has_organizzato_da_esterno and not has_organizzato_da_interno:
            raise BadRequest(
                json.dumps(
                    [
                        {
                            "field": "organizzato_da_esterno",
                            "message": self.mutual_required_msg,
                        },
                        {
                            "field": "organizzato_da_interno",
                            "message": self.mutual_required_msg,
                        },
                        {
                            "message": api.portal.translate(
                                _(
                                    "organizzato_validation_error",
                                    default='Devi compilare almeno uno dei due campi per "Organizzato da" nel tab "Contatti".',
                                )
                            )
                        },
                    ]
                )
            )

        # validate dove
        has_webinar = self.has_field_value(
            data=data, create=create, field_id="webinar", is_block_field=True
        )
        has_struttura_correlata = self.has_field_value(
            data=data, create=create, field_id="struttura_correlata"
        )

        has_location_infos = self.has_location_infos(data=data, create=create)
        if not has_webinar and not has_struttura_correlata and not has_location_infos:
            raise BadRequest(
                json.dumps(
                    [
                        {
                            "field": "webinar",
                            "message": self.mutual_required_msg,
                        },
                        {
                            "field": "struttura_correlata",
                            "message": self.mutual_required_msg,
                        },
                        {
                            "field": "geolocation",
                            "message": self.mutual_required_msg,
                        },
                        {
                            "message": api.portal.translate(
                                _(
                                    "dove_validation_error",
                                    default='Devi compilare almeno uno dei campi obbligatori nel tab "Dove".',
                                )
                            )
                        },
                    ]
                )
            )

    def validate_dove(self, data, create):
        if not self.has_location_infos(data=data, create=create):
            msg = api.portal.translate(
                _(
                    "dove_validation_error",
                    default="Devi compilare questi campi per poter cercare la posizione in mappa.",
                )
            )
            raise BadRequest(
                json.dumps(
                    [
                        {"field": "street", "message": msg},
                        {"field": "city", "message": msg},
                        {
                            "message": api.portal.translate(
                                _(
                                    "dove_validation_error",
                                    default='Devi compilare almeno uno dei due campi del tab "Dove" e impostare la Geolocation.',
                                )
                            )
                        },
                    ]
                )
            )

    def has_location_infos(self, data, create):
        value = self.get_field_value(data, create, "geolocation")
        if not value:
            return False
        if isinstance(value, Geolocation):
            geolocation = {
                "latitude": getattr(value, "latitude", 0),
                "longitude": getattr(value, "longitude", 0),
            }
        else:
            geolocation = deepcopy(value)
        if geolocation in [{"latitude": 0, "longitude": 0}]:
            return False
        return True

    def has_field_value(self, data, create, field_id, is_block_field=False):
        value = self.get_field_value(data, create, field_id)
        if not value:
            return False
        if is_block_field:
            blocks = value.get("blocks", {})
            if not blocks:
                return False
            blocks_data = list(blocks.values())
            for block in blocks_data:
                if block.get("plaintext", ""):
                    # there is some text
                    return True
            return False
        return True

    def get_field_value(self, data, create, field_id):
        if field_id in data:
            return data[field_id]
        if not create:
            return getattr(self.context, field_id, None)
        return None



from iosanita.contenttypes import _
from plone.restapi.controlpanels.interfaces import IControlpanel
from zope.interface import Interface
from zope.schema import Bool


class IIoSanitaContenttypesSettingsControlpanel(IControlpanel):
    """ """


class IIoSanitaContenttypesSettings(Interface):
    """
    Control panel settings
    """

    show_modified_default = Bool(
        title=_("show_modified_default_label", default="Mostra la data di modifica"),
        description=_(
            "show_modified_default_help",
            default="Questo è il valore di default per decidere se mostrare "
            "o meno la data di modifica nei contenuti che hanno la behavior "
            "abilitata. E' poi possibile sovrascrivere il default nei singoli "
            'contenuti (nel tab "Impostazioni").',
        ),
        default=True,
        required=False,
    )



# -*- coding: utf-8 -*-

from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IServizio(model.Schema, IIosanitaContenttypes):
    """Marker interface for content type"""

    servizio_attivo = schema.Bool(
        title=_("servizio_attivo_label", default="Servizio attivo"),
        required=False,
        default=True,
        description=_(
            "servizio_attivo_help",
            default="Indica se il servizio è effettivamente fruibile o meno. "
            "Deselezionare se il servizio non è più attivo.",
        ),
    )

    cosa_serve = BlocksField(
        title=_("cosa_serve_label", default="Cosa serve"),
        required=True,
        description=_(
            "cosa_serve_help",
            default="Descrizione delle istruzioni per usufruire del servizio.",
        ),
    )

    come_accedere = BlocksField(
        title=_("come_accedere_label", default="Come accedere"),
        required=True,
        description=_(
            "come_accedere_servizio_help",
            default="Descrizione della procedura da seguire per poter"
            " usufruire del servizio.",
        ),
    )
    orari = BlocksField(
        title=_("orari_servizio_label", default="Orari del servizio"),
        required=True,
        description=_(
            "orari_servizio_help",
            default="Indicazione dell'orario in cui è possibile usufruire del servizio o della prestazione. "
            "Se gli orari del servizio coincidono con quelli della struttura che lo eroga, ripeterli anche qui.",
        ),
    )

    prenota_online_link = schema.URI(
        title=_("prenota_online_link_label", default="Prenota online"),
        description=_(
            "prenota_online_link_help",
            default="Collegamento con l'eventuale funzionalità di prenotazione online del servizio.",
        ),
        required=False,
    )
    prenota_online_label = schema.TextLine(
        title=_(
            "prenota_online_label_label",
            default="Etichetta bottone per prenota online",
        ),
        description=_(
            "prenota_online_label_help",
            default="Testo da mostrare nel bottone del link per la prenotazione online.",
        ),
        default="Prenota online",
        required=False,
    )
    struttura_correlata = RelationList(
        title=_(
            "struttura_correlata_servizio_label",
            default="Struttura di riferimento",
        ),
        description=_(
            "struttura_correlata_servizio_help",
            default="Seleziona una Struttura o Unità organizzativa di riferimento.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=True,
        missing_value=(),
    )

    tempi_attesa = BlocksField(
        title=_("tempi_attesa_label", default="Tempi di attesa"),
        required=False,
        description=_("tempi_attesa_help", default=""),
    )
    costi = BlocksField(
        title=_("costi_label", default="Costi"),
        required=False,
        description=_("costi_help", default=""),
    )

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa_servizio_label", default="Descrizione estesa"),
        required=False,
        description=_(
            "descrizione_estesa_servizio_help",
            default="Descrizione estesa e completa del servizio o della prestazione. "
            "Se si sta descrivendo un Percorso di Cura specificare contestualmente l'elenco delle prestazioni afferenti al percorso.",
        ),
    )
    procedure_collegate_esito = BlocksField(
        title=_(
            "procedure_collegate_esito_label", default="Procedure collegate all'esito"
        ),
        description=_(
            "procedure_collegate_esito_help",
            default="Spiegazione relativa all'esito della procedura e dove eventualmente "
            "sarà disponibile o sarà possibile ritirare l'esito (sede dell'ufficio, orari, numero sportello, etc.)",
        ),
        required=False,
    )

    uo_correlata = RelationList(
        title=_(
            "uo_correlata_servizio_label", default="Unità organizzativa responsabile"
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        missing_value=(),
    )
    responsabile_correlato = RelationList(
        title=_(
            "responsabile_correlato_servizio_label", default="Responsabile del servizio"
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        missing_value=(),
    )
    servizio_correlato = RelationList(
        title=_("servizio_correlato_label", default="Servizi e prestazioni"),
        description=_(
            "servizio_correlato_servizio_help",
            default="Elenco dei servizi e delle prestazioni dell'ASL correlati a questo.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )
    # widgets
    form.widget(
        "struttura_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Struttura", "UnitaOrganizzativa"],
        },
    )
    form.widget(
        "uo_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "responsabile_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["Persona"],
        },
    )
    form.widget(
        "servizio_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Servizio"],
        },
    )

    # fieldsets
    model.fieldset(
        "cosa_serve",
        label=_("cosa_serve_label", default="Cosa serve"),
        fields=["cosa_serve"],
    )
    model.fieldset(
        "accedi_al_servizio",
        label=_("accedi_al_servizio_label", default="Accedi al servizio"),
        fields=["come_accedere", "prenota_online_link", "prenota_online_label"],
    )
    model.fieldset(
        "tempi_attesa",
        label=_("tempi_attesa_label", default="Tempi di attesa"),
        fields=["tempi_attesa"],
    )
    model.fieldset(
        "costi",
        label=_("costi_label", default="Costi"),
        fields=["costi"],
    )
    model.fieldset(
        "dove",
        label=_("dove_label", default="Dove"),
        fields=["struttura_correlata"],
    )
    model.fieldset(
        "orari",
        label=_("orari_servizio_label", default="Orari del servizio"),
        fields=["orari"],
    )
    model.fieldset(
        "cosa_e",
        label=_("cosa_e_fieldset", default="Cos'è"),
        fields=["descrizione_estesa"],
    )
    model.fieldset(
        "procedure_collegate_esito",
        label=_(
            "procedure_collegate_esito_label", default="Procedure collegate all'esito"
        ),
        fields=["procedure_collegate_esito"],
    )
    model.fieldset(
        "responsabili",
        label=_("responsabili_label", default="Responsabili"),
        fields=["uo_correlata", "responsabile_correlato"],
    )
    model.fieldset(
        "contenuti_collegati",
        label=_("contenuti_collegati_label", default="Contenuti collegati"),
        fields=["servizio_correlato"],
    )
    textindexer.searchable("cosa_serve")



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
# from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from transaction import commit

import unittest


# from Products.CMFPlone.interfaces import ISelectableConstrainTypes


class TestCartellaModulisticaSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.request["LANGUAGE"] = "it"

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.cartella_modulistica = api.content.create(
            container=self.portal, type="CartellaModulistica", title="cm"
        )
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_cartella_modulistica(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["CartellaModulistica"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.categorization",
                "plone.basic",
                "plone.leadimage",
                "volto.preview_image",
                "plone.locking",
                "volto.blocks",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
            ),
        )

    def test_cartella_modulistica_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/CartellaModulistica").json()
        self.assertEqual(len(resp["fieldsets"]), 7)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "settings",
                "ownership",
                "dates",
                "categorization",
                "layout",
                "seo",
            ],
        )

    def test_cartella_modulistica_required_fields(self):
        resp = self.api_session.get("@types/CartellaModulistica").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                ]
            ),
        )

    def test_cartella_modulistica_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/CartellaModulistica").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "anteprima_file",
                "ricerca_in_testata",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
            ],
        )

    def test_cartella_modulistica_expander(self):
        api.content.create(
            container=self.cartella_modulistica, type="File", title="xxx"
        )
        commit()

        resp = self.api_session.get("/cm/@modulistica-items").json()["items"][0]
        self.assertEqual(resp["@type"], "File")
        self.assertEqual(resp["id"], "xxx")



# -*- coding: utf-8 -*-

from . import logger
from iosanita.contenttypes.events.common import SUBFOLDERS_MAPPING
from iosanita.contenttypes.utils import create_default_blocks
from plone import api
from Products.CMFPlone.interfaces import ISelectableConstrainTypes


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Add allegati folder to ComeFarePer ct")

    portal_type = "ComeFarePer"
    mapping = SUBFOLDERS_MAPPING[portal_type]
    children_id = [x["id"] for x in mapping["content"]][0]

    container_brains = api.content.find(portal_type=portal_type)
    tot = len(container_brains)
    logger.info(f"Fixing {tot} {portal_type}")
    for brain in container_brains:
        item = brain.getObject()
        if children_id not in item.keys():
            ptype = mapping.get("type", "Document")
            child = api.content.create(
                container=item,
                type=ptype,
                title=mapping["content"][0]["title"],
                id=children_id,
            )

            create_default_blocks(context=child)

            child.exclude_from_search = True
            child.reindexObject(idxs=["exclude_from_search"])

            child.exclude_from_nav = True
            child.reindexObject(idxs=["exclude_from_nav"])

            if mapping["content"][0].get("allowed_types", ()):
                constraints_child = ISelectableConstrainTypes(child)
                constraints_child.setConstrainTypesMode(1)
                constraints_child.setLocallyAllowedTypes(
                    mapping["content"][0].get("allowed_types", ())
                )



# -*- coding: utf-8 -*-
from iosanita.contenttypes.interfaces.come_fare_per import IComeFarePer
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IComeFarePer)
class ComeFarePer(Container):
    """ """



from collective.taxonomy import PATH_SEPARATOR
from iosanita.contenttypes.indexers.taxonomies import get_taxonomy_vocab
from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.serializer.dxcontent import SerializeFolderToJson
from plone.volto.interfaces import IFolderishNewsItem
from zope.component import adapter
from zope.interface import implementer


@implementer(ISerializeToJson)
@adapter(IFolderishNewsItem, IIosanitaContenttypesLayer)
class NewsItemSerializeToJson(SerializeFolderToJson):
    def __call__(self, version=None, include_items=True):
        data = super().__call__(version=version, include_items=include_items)

        tipologia_notizia = getattr(self.context, "tipologia_notizia")
        taxonomy_vocab = get_taxonomy_vocab("tipologia_notizia")
        taxonomy_value = taxonomy_vocab.inv_data.get(tipologia_notizia, None)
        if taxonomy_value:
            data["type_title"] = taxonomy_value.replace(PATH_SEPARATOR, "", 1)
        else:
            data["type_title"] = "Notizia"
        return data
